@@ -36,137 +36,266 @@ describe("generation", function () {
 
     const generated = generate(combinedAst, { sourceMaps: true }, sources);
 
-    expect(generated.map).toEqual(
-      {
-        version: 3,
-        sources: ["a.js", "b.js"],
-        mappings:
-          "AAAA,SAASA,EAAT,CAAaC,GAAb,EAAkB;EAAEC,OAAO,CAACC,GAAR,CAAYF,GAAZ;AAAmB;;ACAvCD,EAAE,CAAC,OAAD,CAAF",
-        names: ["hi", "msg", "console", "log"],
-        sourcesContent: [
-          "function hi (msg) { console.log(msg); }\n",
-          "hi('hello');\n",
+    expect(generated.map).toMatchInlineSnapshot(`
+      Object {
+        "file": undefined,
+        "mappings": "AAAA,SAASA,EAAE,CAAEC,GAAG,EAAE;EAAEC,OAAO,CAACC,GAAG,CAACF,GAAG,CAAC;AAAE;ACAtCD,EAAE,CAAC,OAAO,CAAC",
+        "names": Array [
+          "hi",
+          "msg",
+          "console",
+          "log",
         ],
-      },
-      "sourcemap was incorrectly generated",
-    );
+        "sourceRoot": undefined,
+        "sources": Array [
+          "a.js",
+          "b.js",
+        ],
+        "sourcesContent": Array [
+          "function hi (msg) { console.log(msg); }
+      ",
+          "hi('hello');
+      ",
+        ],
+        "version": 3,
+      }
+    `);
 
-    expect(generated.rawMappings).toEqual(
-      [
-        {
-          name: undefined,
-          generated: { line: 1, column: 0 },
-          source: "a.js",
-          original: { line: 1, column: 0 },
+    expect(generated.rawMappings).toMatchInlineSnapshot(`
+      Array [
+        Object {
+          "generated": Object {
+            "column": 0,
+            "line": 1,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 0,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: "hi",
-          generated: { line: 1, column: 9 },
-          source: "a.js",
-          original: { line: 1, column: 9 },
+        Object {
+          "generated": Object {
+            "column": 9,
+            "line": 1,
+          },
+          "name": "hi",
+          "original": Object {
+            "column": 9,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 1, column: 11 },
-          source: "a.js",
-          original: { line: 1, column: 0 },
+        Object {
+          "generated": Object {
+            "column": 11,
+            "line": 1,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 11,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: "msg",
-          generated: { line: 1, column: 12 },
-          source: "a.js",
-          original: { line: 1, column: 13 },
+        Object {
+          "generated": Object {
+            "column": 12,
+            "line": 1,
+          },
+          "name": "msg",
+          "original": Object {
+            "column": 13,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 1, column: 15 },
-          source: "a.js",
-          original: { line: 1, column: 0 },
+        Object {
+          "generated": Object {
+            "column": 15,
+            "line": 1,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 16,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 1, column: 17 },
-          source: "a.js",
-          original: { line: 1, column: 18 },
+        Object {
+          "generated": Object {
+            "column": 17,
+            "line": 1,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 18,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: "console",
-          generated: { line: 2, column: 2 },
-          source: "a.js",
-          original: { line: 1, column: 20 },
+        Object {
+          "generated": Object {
+            "column": 2,
+            "line": 2,
+          },
+          "name": "console",
+          "original": Object {
+            "column": 20,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 2, column: 9 },
-          source: "a.js",
-          original: { line: 1, column: 27 },
+        Object {
+          "generated": Object {
+            "column": 9,
+            "line": 2,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 27,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: "log",
-          generated: { line: 2, column: 10 },
-          source: "a.js",
-          original: { line: 1, column: 28 },
+        Object {
+          "generated": Object {
+            "column": 10,
+            "line": 2,
+          },
+          "name": "log",
+          "original": Object {
+            "column": 28,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 2, column: 13 },
-          source: "a.js",
-          original: { line: 1, column: 20 },
+        Object {
+          "generated": Object {
+            "column": 13,
+            "line": 2,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 31,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: "msg",
-          generated: { line: 2, column: 14 },
-          source: "a.js",
-          original: { line: 1, column: 32 },
+        Object {
+          "generated": Object {
+            "column": 14,
+            "line": 2,
+          },
+          "name": "msg",
+          "original": Object {
+            "column": 32,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 2, column: 17 },
-          source: "a.js",
-          original: { line: 1, column: 20 },
+        Object {
+          "generated": Object {
+            "column": 17,
+            "line": 2,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 35,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 3, column: 0 },
-          source: "a.js",
-          original: { line: 1, column: 39 },
+        Object {
+          "generated": Object {
+            "column": 18,
+            "line": 2,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 36,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: "hi",
-          generated: { line: 5, column: 0 },
-          source: "b.js",
-          original: { line: 1, column: 0 },
+        Object {
+          "generated": Object {
+            "column": 0,
+            "line": 3,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 38,
+            "line": 1,
+          },
+          "source": "a.js",
         },
-        {
-          name: undefined,
-          generated: { line: 5, column: 2 },
-          source: "b.js",
-          original: { line: 1, column: 2 },
+        Object {
+          "generated": Object {
+            "column": 0,
+            "line": 4,
+          },
+          "name": "hi",
+          "original": Object {
+            "column": 0,
+            "line": 1,
+          },
+          "source": "b.js",
         },
-        {
-          name: undefined,
-          generated: { line: 5, column: 3 },
-          source: "b.js",
-          original: { line: 1, column: 3 },
+        Object {
+          "generated": Object {
+            "column": 2,
+            "line": 4,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 2,
+            "line": 1,
+          },
+          "source": "b.js",
         },
-        {
-          name: undefined,
-          generated: { line: 5, column: 10 },
-          source: "b.js",
-          original: { line: 1, column: 2 },
+        Object {
+          "generated": Object {
+            "column": 3,
+            "line": 4,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 3,
+            "line": 1,
+          },
+          "source": "b.js",
         },
-        {
-          name: undefined,
-          generated: { line: 5, column: 11 },
-          source: "b.js",
-          original: { line: 1, column: 0 },
+        Object {
+          "generated": Object {
+            "column": 10,
+            "line": 4,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 10,
+            "line": 1,
+          },
+          "source": "b.js",
         },
-      ],
-      "raw mappings were incorrectly generated",
-    );
+        Object {
+          "generated": Object {
+            "column": 11,
+            "line": 4,
+          },
+          "name": undefined,
+          "original": Object {
+            "column": 11,
+            "line": 1,
+          },
+          "source": "b.js",
+        },
+      ]
+    `);
 
     expect(generated.code).toBe(
-      "function hi(msg) {\n  console.log(msg);\n}\n\nhi('hello');",
+      "function hi(msg) {\n  console.log(msg);\n}\nhi('hello');",
     );
   });
 
@@ -199,7 +328,7 @@ describe("generation", function () {
         version: 3,
         sources: ["inline"],
         names: ["foo", "bar"],
-        mappings: "AAAA,SAASA,IAAT,GAAe;EAAEC,IAAG;AAAG",
+        mappings: "AAAA,SAASA,IAAG,GAAG;EAAEC,IAAG;AAAE",
         sourcesContent: ["function foo() { bar; }\n"],
       },
       "sourcemap was incorrectly generated",
@@ -223,7 +352,7 @@ describe("generation", function () {
           name: undefined,
           generated: { line: 1, column: 13 },
           source: "inline",
-          original: { line: 1, column: 0 },
+          original: { line: 1, column: 12 },
         },
         {
           name: undefined,
@@ -247,7 +376,7 @@ describe("generation", function () {
           name: undefined,
           generated: { line: 3, column: 0 },
           source: "inline",
-          original: { line: 1, column: 23 },
+          original: { line: 1, column: 22 },
         },
       ],
       "raw mappings were incorrectly generated",
@@ -341,9 +470,230 @@ describe("generation", function () {
 
     const ast = t.program([...ast1.body, ...ast2.body]);
 
-    expect(generate(ast).code).toBe(
-      "/*#__PURE__*/\na();\n\n/*#__PURE__*/\nb();",
+    expect(generate(ast).code).toBe("/*#__PURE__*/a();\n/*#__PURE__*/b();");
+  });
+
+  it("comments with null or undefined loc", () => {
+    const code = "/*#__PURE__*/ /*#__PURE__*/";
+
+    const ast = parse(code);
+
+    ast.comments[0].loc = null;
+    ast.comments[1].loc = undefined;
+
+    expect(generate(ast).code).toBe("/*#__PURE__*/\n/*#__PURE__*/");
+  });
+
+  it("comments without loc", () => {
+    const ast = parse(
+      `
+        import {
+            Attribute,
+            AttributeSDKType
+        }
+        from "../../base/v1beta1/attribute";
+        import {
+            Rpc
+        }
+        from "../../../helpers";
+        import * as _m0 from "protobufjs/minimal";
+        import {
+            MsgSignProviderAttributes,
+            MsgSignProviderAttributesSDKType,
+            MsgSignProviderAttributesResponse,
+            MsgSignProviderAttributesResponseSDKType,
+            MsgDeleteProviderAttributes,
+            MsgDeleteProviderAttributesSDKType,
+            MsgDeleteProviderAttributesResponse,
+            MsgDeleteProviderAttributesResponseSDKType
+        }
+        from "./audit";
+        /** Msg defines the provider Msg service */
+        export interface Msg {
+            /** SignProviderAttributes defines a method that signs provider attributes */
+            signProviderAttributes(request: MsgSignProviderAttributes): Promise < MsgSignProviderAttributesResponse > ;
+            /** DeleteProviderAttributes defines a method that deletes provider attributes */
+            deleteProviderAttributes(request: MsgDeleteProviderAttributes): Promise < MsgDeleteProviderAttributesResponse > ;
+        }
+        export class MsgClientImpl implements Msg {
+            private readonly rpc: Rpc;
+            constructor(rpc: Rpc) {
+                this.rpc = rpc;
+            }
+            /* SignProviderAttributes defines a method that signs provider attributes */
+            signProviderAttributes = async(request: MsgSignProviderAttributes): Promise < MsgSignProviderAttributesResponse >  => {
+                const data = MsgSignProviderAttributes.encode(request).finish();
+                const promise = this.rpc.request("akash.audit.v1beta1.Msg", "SignProviderAttributes", data);
+                return promise.then(data => MsgSignProviderAttributesResponse.decode(new _m0.Reader(data)));
+            };
+            /* DeleteProviderAttributes defines a method that deletes provider attributes */
+            deleteProviderAttributes = async(request: MsgDeleteProviderAttributes): Promise < MsgDeleteProviderAttributesResponse >  => {
+                const data = MsgDeleteProviderAttributes.encode(request).finish();
+                const promise = this.rpc.request("akash.audit.v1beta1.Msg", "DeleteProviderAttributes", data);
+                return promise.then(data => MsgDeleteProviderAttributesResponse.decode(new _m0.Reader(data)));
+            };
+        }
+    `,
+      { sourceType: "module", plugins: ["typescript"] },
     );
+
+    for (const comment of ast.comments) {
+      comment.loc = undefined;
+    }
+
+    expect(generate(ast).code).toMatchInlineSnapshot(`
+      "import { Attribute, AttributeSDKType } from \\"../../base/v1beta1/attribute\\";
+      import { Rpc } from \\"../../../helpers\\";
+      import * as _m0 from \\"protobufjs/minimal\\";
+      import { MsgSignProviderAttributes, MsgSignProviderAttributesSDKType, MsgSignProviderAttributesResponse, MsgSignProviderAttributesResponseSDKType, MsgDeleteProviderAttributes, MsgDeleteProviderAttributesSDKType, MsgDeleteProviderAttributesResponse, MsgDeleteProviderAttributesResponseSDKType } from \\"./audit\\";
+      /** Msg defines the provider Msg service */
+      export interface Msg {
+        /** SignProviderAttributes defines a method that signs provider attributes */
+        signProviderAttributes(request: MsgSignProviderAttributes): Promise<MsgSignProviderAttributesResponse>;
+        /** DeleteProviderAttributes defines a method that deletes provider attributes */
+        deleteProviderAttributes(request: MsgDeleteProviderAttributes): Promise<MsgDeleteProviderAttributesResponse>;
+      }
+      export class MsgClientImpl implements Msg {
+        private readonly rpc: Rpc;
+        constructor(rpc: Rpc) {
+          this.rpc = rpc;
+        }
+        /* SignProviderAttributes defines a method that signs provider attributes */
+        signProviderAttributes = async (request: MsgSignProviderAttributes): Promise<MsgSignProviderAttributesResponse> => {
+          const data = MsgSignProviderAttributes.encode(request).finish();
+          const promise = this.rpc.request(\\"akash.audit.v1beta1.Msg\\", \\"SignProviderAttributes\\", data);
+          return promise.then(data => MsgSignProviderAttributesResponse.decode(new _m0.Reader(data)));
+        };
+        /* DeleteProviderAttributes defines a method that deletes provider attributes */
+        deleteProviderAttributes = async (request: MsgDeleteProviderAttributes): Promise<MsgDeleteProviderAttributesResponse> => {
+          const data = MsgDeleteProviderAttributes.encode(request).finish();
+          const promise = this.rpc.request(\\"akash.audit.v1beta1.Msg\\", \\"DeleteProviderAttributes\\", data);
+          return promise.then(data => MsgDeleteProviderAttributesResponse.decode(new _m0.Reader(data)));
+        };
+      }"
+    `);
+  });
+
+  it("comments without loc2", () => {
+    const ast = parse(
+      `
+        (function (_templateFactory) {
+            "use strict";
+
+            const template = (0, _templateFactory.createTemplateFactory)(
+            /*{{somevalue}}*/
+            {
+                "id": null,
+                "block": "[[[1,[34,0]]],[],false,[\\"somevalue\\"]]",
+                "moduleName": "(unknown template module)",
+                "isStrictMode": false
+            });
+        });
+
+        const template = (0, _templateFactory.createTemplateFactory)(
+        /*
+          {{somevalue}}
+        */
+        {
+            "id": null,
+            "block": "[[[1,[34,0]]],[],false,[\\"somevalue\\"]]",
+            "moduleName": "(unknown template module)",
+            "isStrictMode": false
+        });
+      `,
+      { sourceType: "module" },
+    );
+
+    for (const comment of ast.comments) {
+      comment.loc = undefined;
+    }
+
+    expect(generate(ast).code).toMatchInlineSnapshot(`
+      "(function (_templateFactory) {
+        \\"use strict\\";
+
+        const template = (0, _templateFactory.createTemplateFactory)(
+        /*{{somevalue}}*/
+        {
+          \\"id\\": null,
+          \\"block\\": \\"[[[1,[34,0]]],[],false,[\\\\\\"somevalue\\\\\\"]]\\",
+          \\"moduleName\\": \\"(unknown template module)\\",
+          \\"isStrictMode\\": false
+        });
+      });
+      const template = (0, _templateFactory.createTemplateFactory)(
+      /*
+                {{somevalue}}
+              */
+      {
+        \\"id\\": null,
+        \\"block\\": \\"[[[1,[34,0]]],[],false,[\\\\\\"somevalue\\\\\\"]]\\",
+        \\"moduleName\\": \\"(unknown template module)\\",
+        \\"isStrictMode\\": false
+      });"
+    `);
+  });
+
+  it("comments without node.loc", () => {
+    const ast = parse(
+      `
+        (function (_templateFactory) {
+            "use strict";
+
+            const template = (0, _templateFactory.createTemplateFactory)(
+            /*{{somevalue}}*/
+            {
+                "id": null,
+                "block": "[[[1,[34,0]]],[],false,[\\"somevalue\\"]]",
+                "moduleName": "(unknown template module)",
+                "isStrictMode": false
+            });
+        });
+
+        const template = (0, _templateFactory.createTemplateFactory)(
+        /*
+          {{somevalue}}
+        */
+        {
+            "id": null,
+            "block": "[[[1,[34,0]]],[],false,[\\"somevalue\\"]]",
+            "moduleName": "(unknown template module)",
+            "isStrictMode": false
+        });
+      `,
+      { sourceType: "module" },
+    );
+
+    const ast2 = t.cloneNode(ast, true, true);
+
+    for (let i = 0; i < ast.comments.length; i++) {
+      ast2.comments[i].loc = ast.comments[i].loc;
+    }
+
+    expect(generate(ast2).code).toMatchInlineSnapshot(`
+      "(function (_templateFactory) {
+        \\"use strict\\";
+
+        const template = (0, _templateFactory.createTemplateFactory)(
+        /*{{somevalue}}*/
+        {
+          \\"id\\": null,
+          \\"block\\": \\"[[[1,[34,0]]],[],false,[\\\\\\"somevalue\\\\\\"]]\\",
+          \\"moduleName\\": \\"(unknown template module)\\",
+          \\"isStrictMode\\": false
+        });
+      });
+      const template = (0, _templateFactory.createTemplateFactory)(
+      /*
+        {{somevalue}}
+      */
+      {
+        \\"id\\": null,
+        \\"block\\": \\"[[[1,[34,0]]],[],false,[\\\\\\"somevalue\\\\\\"]]\\",
+        \\"moduleName\\": \\"(unknown template module)\\",
+        \\"isStrictMode\\": false
+      });"
+    `);
   });
 });
 
@@ -845,7 +1195,7 @@ suites.forEach(function (testSuite) {
                 try {
                   expect(result.map).toEqual(task.sourceMap);
                 } catch (e) {
-                  if (!process.env.OVERWRITE || !task.sourceMapFile) throw e;
+                  if (!process.env.OVERWRITE && task.sourceMap) throw e;
                   console.log(`Updated test file: ${task.sourceMapFile.loc}`);
                   fs.writeFileSync(
                     task.sourceMapFile.loc,
