@@ -222,7 +222,19 @@ export default abstract class StatementParser extends ExpressionParser {
         this.raise(Errors.ModuleExportUndefined, { at, localName });
       }
     }
-    return this.finishNode(program, "Program");
+    let finishedProgram: N.Program;
+    if (end === tt.eof) {
+      // finish at eof for top level program
+      finishedProgram = this.finishNode(program, "Program");
+    } else {
+      // finish immediately before the end token
+      finishedProgram = this.finishNodeAt(
+        program,
+        "Program",
+        createPositionWithColumnOffset(this.state.startLoc, -1),
+      );
+    }
+    return finishedProgram;
   }
 
   // TODO
@@ -570,17 +582,15 @@ export default abstract class StatementParser extends ExpressionParser {
       // So that the decorators of any nested class expressions will be dealt with separately
       this.state.decoratorStack.push([]);
 
-      const startPos = this.state.start;
       const startLoc = this.state.startLoc;
       let expr: N.Expression;
 
       if (this.match(tt.parenL)) {
-        const startPos = this.state.start;
         const startLoc = this.state.startLoc;
         this.next(); // eat '('
         expr = this.parseExpression();
         this.expect(tt.parenR);
-        expr = this.wrapParenthesis(startPos, startLoc, expr);
+        expr = this.wrapParenthesis(startLoc, expr);
 
         const paramsStartLoc = this.state.startLoc;
         node.expression = this.parseMaybeDecoratorArguments(expr);
@@ -597,7 +607,7 @@ export default abstract class StatementParser extends ExpressionParser {
         expr = this.parseIdentifier(false);
 
         while (this.eat(tt.dot)) {
-          const node = this.startNodeAt(startPos, startLoc);
+          const node = this.startNodeAt(startLoc);
           node.object = expr;
           if (this.match(tt.privateName)) {
             this.classScope.usePrivateName(
@@ -2197,10 +2207,7 @@ export default abstract class StatementParser extends ExpressionParser {
     if (this.isContextual(tt._as)) {
       if (!node.specifiers) node.specifiers = [];
 
-      const specifier = this.startNodeAt(
-        this.state.lastTokStart,
-        this.state.lastTokStartLoc,
-      );
+      const specifier = this.startNodeAt(this.state.lastTokStartLoc);
 
       this.next();
 
