@@ -1,10 +1,8 @@
 "use strict";
 
 let called = false;
-
 let Base = /*#__PURE__*/function () {
   function Base() {}
-
   babelHelpers.createClass(Base, [{
     key: "test",
     get: function () {
@@ -14,35 +12,30 @@ let Base = /*#__PURE__*/function () {
   }]);
   return Base;
 }();
-
 ;
-
 let Obj = /*#__PURE__*/function (_Base) {
   babelHelpers.inheritsLoose(Obj, _Base);
-
   function Obj() {
     return _Base.apply(this, arguments) || this;
   }
-
   var _proto = Obj.prototype;
-
   _proto.set = function set() {
     return this.test = 3;
   };
-
   return Obj;
 }(Base);
-
 Object.defineProperty(Obj.prototype, 'test', {
   value: 2,
   writable: true,
   configurable: true
 });
-const obj = new Obj(); // This is incorrect according to the spec,
+const obj = new Obj();
+
+// This is incorrect according to the spec,
 // but close enough for loose.
 // expect(() => {
-
-expect(obj.set()).toBe(3); // }).toThrow();
+expect(obj.set()).toBe(3);
+// }).toThrow();
 
 expect(called).toBe(false);
 expect(Base.prototype.test).toBe(1);
